/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "engine.h"

engine_t * engineInit(char *gameName) {
  // Create the engine instance.
  engine_t *engine = malloc(sizeof(engine_t));
  if(engine == NULL) return NULL;

  // Setup the renderer.
  engine->render = renderInit(gameName);
  if(engine->render == NULL) {
    free(engine);
    return NULL;
  }

  // Setup the input manager.
  engine->input = inputInit(0);
  if(engine->input == NULL) {
    free(engine->render);
    free(engine);
    return NULL;
  }

  return engine;
}

void engineStart(engine_t *engine) {
  while(!glfwWindowShouldClose(engine->render->window)) {
    inputUpdate(engine->input);
    renderFrame(engine->render);
    glfwSwapBuffers(engine->render->window);
    glfwPollEvents();
  }
}

bool engineDispose(engine_t *engine) {
  if(!renderDispose(engine->render)) return false;
  free(engine);

  return true;
}