// Copyright (c) 2021 Dominic Msters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <stdbool.h>
#include "../display/render.h"
#include "../file/asset.h"
#include "../input/input.h"

/** Information about the current engine context. */
typedef struct {
  /** Name of the game running. */
  char *gameName;

  /** Renderer for the engine. */
  render_t *render;
  
  /** Input Manager for the engine. */
  input_t *input;
} engine_t;

/**
 * Initialize the engine context.
 * 
 * @param gameName Name of the game being initialized.
 * @return The engine context.
 */
engine_t * engineInit(char *gameName);

/**
 * Start the main engine loop.
 * 
 * @param engine The game to start the loop for.
 */
void engineStart(engine_t *engine);

/**
 * Cleanup a previously constructed game engine instance.
 * 
 * @param engine The engine to cleanup.
 * @return True if successful or not.
 */
bool engineDispose(engine_t *engine);

