/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "input.h"

input_t * inputInit(uint32_t inputBindCount) {
  input_t *input = malloc(sizeof(input_t));
  if(!input) return NULL;
  input->inputBindCount = inputBindCount;

  input->inputsCurrent = malloc(sizeof(float) * inputBindCount);
  if(input->inputsCurrent == NULL) {
    free(input);
    return NULL;
  }

  input->inputsPrevious = malloc(sizeof(float) * inputBindCount);
  if(input->inputsPrevious == NULL) {
    free(input->inputsCurrent);
    free(input);
    return NULL;
  }

  input->inputTime = malloc(sizeof(float) * inputBindCount);
  if(input->inputTime == NULL) {
    free(input->inputsPrevious);
    free(input->inputsCurrent);
    free(input);
    return NULL;
  }

  return input;
}

void inputUpdate(input_t *input) {
  // Flip the states to save memory.
  float * currentCurrent = input->inputsCurrent;
  input->inputsCurrent = input->inputsPrevious;
  input->inputsPrevious = currentCurrent;
}

void inputDispose(input_t *input) {
  free(input->inputTime);
  free(input->inputsPrevious);
  free(input->inputsCurrent);
  free(input);
}

inputbindmap_t inputBind(input_t *input, inputbind_t bind,
  inputsource_t *source
) {
  return NULL;
}

float inputGetState(inputbind_t binding, inputsource_t *source) {
  return 0;
}

bool inputIsDown(input_t *input, inputbind_t binding) {
  return input->inputsCurrent[binding] != 0;
}

bool inputIsUp(input_t *input, inputbind_t binding) {
  return input->inputsCurrent[binding] == 0;
}

bool inputIsPressed(input_t *input, inputbind_t binding) {
  return (
    input->inputsPrevious[binding] == 0 &&
    input->inputsCurrent[binding] != 0
  );
}

bool inputIsReleased(input_t *input, inputbind_t binding) {
  return (
    input->inputsCurrent[binding] == 0 &&
    input->inputsPrevious[binding] != 0
  );
}

float inputGetAxis(input_t *input, inputbind_t binding) {
  return input->inputsCurrent[binding];
}

float inputGetFullAxis(input_t *input, inputbind_t positive,
  inputbind_t negative
) {
  return input->inputsCurrent[negative] + input->inputsCurrent[positive];
}

float inputGetAccuated(input_t *input, inputbind_t binding) {
  return input->inputTime[binding];
}